﻿using AdvanceSoftware.VBReport;
using System.Data;

namespace VBR11WebSample.Models
{
    public class ReportService
    {
        private readonly string templatePath;

        public ReportService(IWebHostEnvironment environment)
        {
            templatePath = Path.Combine(environment.WebRootPath, "resource", "template.xlsx");
        }

        private DataTable GetData()
        {
            DataTable dataTable = new DataTable();
            dataTable.Columns.Add("**Company", typeof(string));
            dataTable.Columns.Add("**Delivery", typeof(string));
            dataTable.Columns.Add("**Item", typeof(string));
            dataTable.Columns.Add("**Quantity", typeof(int));
            dataTable.Columns.Add("**UnitPrice", typeof(int));
            dataTable.Columns.Add("**Price", typeof(int));
            dataTable.Rows.Add("○○株式会社", "西部配送センター", "ハウスキット１", 5, 3080, 15400);
            dataTable.Rows.Add("○○株式会社", "西部配送センター", "ハウスキット２", 5, 1760, 8800);
            dataTable.Rows.Add("○○株式会社", "西部配送センター", "プッシュ式型抜きセット", 10, 110, 1100);
            dataTable.Rows.Add("○○株式会社", "西部配送センター", "クレイガン", 5, 1980, 9900);
            dataTable.Rows.Add("○○株式会社", "西部配送センター", "ハウスキット３", 5, 1540, 7700);
            dataTable.Rows.Add("○○株式会社", "西部配送センター", "樹脂粘土", 100, 660, 66000);
            dataTable.Rows.Add("○○株式会社", "西部配送センター", "カラーフェルト20cm 5色", 150, 300, 45000);
            dataTable.Rows.Add("○○株式会社", "西部配送センター", "ぬい針3本セット", 100, 200, 20000);
            dataTable.Rows.Add("○○株式会社", "東部配送センター", "ハウスキット４", 3, 1000, 3000);
            dataTable.Rows.Add("○○株式会社", "東部配送センター", "ハウスキット５", 10, 4500, 45000);
            dataTable.Rows.Add("○○株式会社", "東部配送センター", "プッシュ式型抜きセット２", 10, 150, 1500);
            dataTable.Rows.Add("○○株式会社", "東部配送センター", "クレイガン", 5, 1980, 9900);
            dataTable.Rows.Add("○○株式会社", "東部配送センター", "ハウスキット３", 5, 1540, 7700);
            dataTable.Rows.Add("○○株式会社", "南部配送センター", "樹脂粘土", 100, 660, 66000);
            dataTable.Rows.Add("○○株式会社", "南部配送センター", "カラーフェルト30cm 3色", 180, 323, 58140);
            dataTable.Rows.Add("○○株式会社", "南部配送センター", "ぬい針6本セット", 200, 385, 77000);
            dataTable.Rows.Add("○○株式会社", "南部配送センター", "ハウスキット６", 5, 6000, 30000);
            dataTable.Rows.Add("○○株式会社", "北部配送センター", "ハウスキット２", 5, 1760, 8800);
            dataTable.Rows.Add("○○株式会社", "北部配送センター", "プッシュ式型抜きセット３", 20, 110, 2200);
            dataTable.Rows.Add("○○株式会社", "北部配送センター", "クレイガン", 5, 1980, 9900);
            dataTable.Rows.Add("○○株式会社", "北部配送センター", "ハウスキット３", 5, 1540, 7700);
            dataTable.Rows.Add("○○株式会社", "北部配送センター", "樹脂粘土", 100, 660, 66000);
            dataTable.Rows.Add("○○株式会社", "北部配送センター", "カラーフェルト30cm 5色", 200, 341, 68200);
            dataTable.Rows.Add("○○株式会社", "北部配送センター", "ぬい針12本セット", 400, 200, 80000);
            return dataTable;
        }

        public string DataList()
        {
            using (CellReport cellReport1 = new CellReport())
            {
                DataTable dataTable = GetData();
                int row = 0;
                cellReport1.Report.Start();
                // Create メソッドで帳票の新規作成を開始、第 1 引数には Excel バージョンを指定
                cellReport1.Report.Create(AdvanceSoftware.VBReport.ExcelVersion.ver2021);
                cellReport1.Page.Start("Sheet1", "1");
                // セル幅の調整
                cellReport1.Cell("A").ColWidth = 12;
                cellReport1.Cell("B").ColWidth = 15;
                cellReport1.Cell("C").ColWidth = 21;
                cellReport1.Cell("D").ColWidth = 9;
                cellReport1.Cell("E").ColWidth = 10;
                cellReport1.Cell("F").ColWidth = 9;
                // データを追加
                for (int column = 0; column < dataTable.Columns.Count; column++)
                {
                    cellReport1.Pos(column, row).Value = dataTable.Columns[column].ColumnName;
                }
                row++;
                foreach (DataRow dataRow in dataTable.Rows)
                {
                    for (int column = 0; column < dataTable.Columns.Count; column++)
                    {
                        cellReport1.Pos(column, row).Value = dataRow[column];
                    }
                    row++;
                }
                cellReport1.Page.End();
                cellReport1.Report.End();
                return cellReport1.Report.GetSvgzReport(AdvanceSoftware.VBReport.SvgSaveType.IncludeExcelPdf);
            }
        }

        public string Invoice()
        {
            using (CellReport cellReport1 = new CellReport())
            {
                DataTable dataTable = GetData();
                // テンプレートファイルのパスを指定
                cellReport1.FileName = templatePath;
                cellReport1.ApplyFormula = true;
                // 帳票ドキュメントの作成を開始
                cellReport1.Report.Start();
                // テンプレートファイルを読み込み
                cellReport1.Report.File();
                cellReport1.Page.Start("納品書", "1");
                var deliveryGroups = dataTable.AsEnumerable().GroupBy(dataRow => dataRow.Field<string>("**Delivery"));
                foreach (var delivery in deliveryGroups)
                {
                    InvoiceDetail(cellReport1, delivery.ToArray());
                }
                cellReport1.Page.End();
                cellReport1.Report.End();
                return cellReport1.Report.GetSvgzReport(AdvanceSoftware.VBReport.SvgSaveType.IncludeExcelPdf);
            }
        }

        private void InvoiceDetail(CellReport cellReport1, DataRow[] delivery)
        {
            const int TEMPLATE_DETAILROW_INDEX = 10;
            int detailCount = 0;
            int detailTotalCount = 0;
            // 明細作成
            foreach (DataRow dataRow in delivery)
            {
                if (detailCount == 0)
                {
                    cellReport1.Page.Repeat(1);
                    cellReport1.Cell("**Company").Value = dataRow["**Company"];
                    cellReport1.Cell("**Delivery").Value = dataRow["**Delivery"];
                    cellReport1.RowCopy(TEMPLATE_DETAILROW_INDEX);
                }
                detailCount++;
                detailTotalCount++;
                cellReport1.RowInsertCopied(TEMPLATE_DETAILROW_INDEX + detailCount, 1);
                cellReport1.Cell("**Item", 0, detailCount).Value = dataRow["**Item"];
                cellReport1.Cell("**Quantity", 0, detailCount).Value = dataRow["**Quantity"];
                cellReport1.Cell("**UnitPrice", 0, detailCount).Value = dataRow["**UnitPrice"];
                cellReport1.Cell("**Price", 0, detailCount).Value = dataRow["**Price"];
                if (detailCount == 5 || delivery.Count() == detailTotalCount)
                {
                    cellReport1.RowDelete(TEMPLATE_DETAILROW_INDEX, 1);
                    cellReport1.Cell("**TotalPrice").Func("=SUM(J11:J" + (TEMPLATE_DETAILROW_INDEX + detailCount) + ")", null);
                    detailCount = 0;
                    cellReport1.Page.Next(true);
                }
            }
        }
    }
}
